%if 0%{?fedora}
%global with_devel 0
%global with_bundled 1
%global with_debug 1
%else
%global with_devel 0
%global with_bundled 1
%global with_debug 0
%endif

%if 0%{?with_debug}
# https://bugzilla.redhat.com/show_bug.cgi?id=995136#c12
%global _dwz_low_mem_die_limit 0
%else
%global debug_package   %{nil}
%endif

%global k8s_repo            kubernetes

#I really need this, otherwise "version_ldflags=$(kube::version_ldflags)"
# does not work
%global _buildshell /bin/bash
%global _checkshell /bin/bash

# k8s release version major.minor
%global k8s_major_version 1
%global k8s_minor_version 12
%global k8s_patch_version 10

%global cveimage_version %{version}-3

# GOLANG version
%global golang_version 1.12.10

Name:          kubernetes
Version:       %{k8s_major_version}.%{k8s_minor_version}.%{k8s_patch_version}
Release:       1.0.13%{?dist}
Summary:       Container cluster management
License:       ASL 2.0
Group:         System/Management
URL:           https://kubernetes.io
Vendor:        Oracle America
ExclusiveArch: x86_64 ppc64le %{arm} aarch64
Source:        kubernetes-%{version}.tar.bz2
Source31:      genmanpages.sh

Source40:      10-kubeadm.conf
Source41:      kubelet.service
Source42:      kubeadm-setup.sh
Source43:      k8s.conf
Source45:      flannel-ol.yaml
Source46:      kubectl-proxy.service
Source47:      10-kubectl-proxy.conf
Source48:      kubernetes-dashboard-self-certs.yaml
Source50:      kubeadm-registry.sh
Source51:      kubelet

%description
%{summary}

BuildRequires: golang == %{golang_version}
BuildRequires: systemd
BuildRequires: rsync
BuildRequires: go-md2man
BuildRequires: git

%package -n kubelet
Summary: Container cluster management
BuildRequires: golang == %{golang_version}
BuildRequires: systemd
BuildRequires: curl
BuildRequires: go-md2man
Requires: iptables >= 1.4.21
Requires: socat
Requires: util-linux
Requires: ethtool
Requires: iproute
Requires: ebtables
Requires: conntrack

%description -n kubelet
The node agent of Kubernetes, the container cluster manager.

%package -n kubectl
Summary: Command-line utility for interacting with a Kubernetes cluster.

%description -n kubectl
Command-line utility for interacting with a Kubernetes cluster.

%package -n kubeadm
Summary: Command-line utility for administering a Kubernetes cluster.
Requires: kubernetes-cni >= 0.7.1
Requires: docker-engine >= 1.12.6
Requires: openssl

%description -n kubeadm
Command-line utility for administering a Kubernetes cluster.

%package -n kubeadm-container-images
Summary: Contains Oracle built k8s docker images

%description -n kubeadm-container-images
Contains Oracle built k8s docker images

%prep
%setup -q -n %{k8s_repo}-%{version}

dirs=$(ls | grep -v "^Godeps")

# Move all the code under src/k8s.io/kubernetes directory
mkdir -p src/k8s.io/kubernetes
mv $(ls | grep -v "^src$") src/k8s.io/kubernetes/.

%build
pushd src/k8s.io/kubernetes/
export KUBE_EXTRA_GOPATH=$(pwd)/Godeps/_workspace

export GOPATH=$(pwd)/Godeps/_workspace
export KUBE_GIT_TREE_STATE=clean
export KUBE_GIT_VERSION=v%{version}+%{release}
export KUBE_GIT_MAJOR=%{k8s_major_version}
export KUBE_GIT_MINOR=%{k8s_minor_version}

go version
go get -u github.com/jteeuwen/go-bindata/go-bindata

make WHAT='cmd/kube-proxy cmd/kube-apiserver cmd/kube-controller-manager cmd/kubelet cmd/hyperkube cmd/kube-scheduler cmd/kubectl cmd/kubeadm'

chmod +x build-k8s-docker.sh
./build-k8s-docker.sh %{cveimage_version} _output/bin/ %{golang_version}
# convert md to man
pushd docs
pushd admin
cp kube-apiserver.md kube-controller-manager.md kube-proxy.md kube-scheduler.md kubelet.md ..
popd
cp %{SOURCE31} genmanpages.sh
bash genmanpages.sh
popd
popd

%install
pushd src/k8s.io/kubernetes/
. hack/lib/init.sh

%ifarch ppc64le
output_path="${KUBE_OUTPUT_BINPATH}"
%else
output_path="${KUBE_OUTPUT_BINPATH}/linux/amd64"
%endif

binaries=(kubelet kubectl kubeadm)
install -m 755 -d %{buildroot}%{_bindir}
for bin in "${binaries[@]}"; do
  echo "+++ INSTALLING ${bin}"
  install -p -m 755 -t %{buildroot}%{_bindir} ${output_path}/${bin}
done

# install the bash completion
install -d -m 0755 %{buildroot}%{_datadir}/bash-completion/completions/
%{buildroot}%{_bindir}/kubectl completion bash > %{buildroot}%{_datadir}/bash-completion/completions/kubectl

# install manpages
install -d %{buildroot}%{_mandir}/man1
install -p -m 644 docs/man/man1/kubectl* %{buildroot}%{_mandir}/man1
install -p -m 644 docs/man/man1/kubelet* %{buildroot}%{_mandir}/man1
install -p -m 644 docs/man/man1/kubeadm* %{buildroot}%{_mandir}/man1
# from k8s tarball copied docs/man/man1/*.1

# EXTRA FOR KUBEADM/KUBELET
install -m 755 -d %{buildroot}%{_sysconfdir}/systemd/system/kubelet.service.d/
install -p -m 644 -t %{buildroot}%{_sysconfdir}/systemd/system/kubelet.service.d %{SOURCE40}
install -m 755 -d %{buildroot}%{_sysconfdir}/systemd/system/kubectl-proxy.service.d/
install -p -m 644 -t %{buildroot}%{_sysconfdir}/systemd/system/kubectl-proxy.service.d %{SOURCE47}

install -m 755 -d %{buildroot}%{_sysconfdir}/kubernetes/manifests/
install -m 755 -d %{buildroot}%{_sysconfdir}/systemd/system/
install -m 755 -d %{buildroot}/usr/libexec/kubernetes/kubelet-plugins/volume/exec/

install -p -m 644 -t %{buildroot}%{_sysconfdir}/systemd/system/ %{SOURCE41}
install -p -m 644 -t %{buildroot}%{_sysconfdir}/systemd/system/ %{SOURCE46}
install -p -m 755 -t %{buildroot}%{_bindir} %{SOURCE42}
install -p -m 755 -t %{buildroot}%{_bindir} %{SOURCE50}
install -d -m 755 %{buildroot}/etc/sysctl.d
install -p -m 644 -t %{buildroot}/etc/sysctl.d %{SOURCE43}

install -d -m 755 %{buildroot}/etc/sysconfig
install -p -m 755 -t %{buildroot}/etc/sysconfig %{SOURCE51}

# EXTRA FOR DOCKER IMAGES
mkdir -p %{buildroot}/usr/local/share/kubeadm
install -m 755 -d %{buildroot}/usr/local/share/kubeadm
images=(kube-apiserver.tar kube-controller-manager.tar kube-scheduler.tar kube-proxy.tar pause.tar)
for bin in "${images[@]}"; do
  echo "+++ INSTALLING DOCKER IMAGES ${bin}"
  install -p -m 755 -t %{buildroot}/usr/local/share/kubeadm ${output_path}/oracle_docker/${bin}
done

# INSTALL K8S RELATED YAML
install -p -m 644 -t %{buildroot}/usr/local/share/kubeadm %{SOURCE45}
install -p -m 644 -t %{buildroot}/usr/local/share/kubeadm %{SOURCE48}

popd
mv src/k8s.io/kubernetes/*.md .
mv src/k8s.io/kubernetes/LICENSE .
mv src/k8s.io/kubernetes/THIRD_PARTY_LICENSES.txt .

%check
#define license tag if not already defined
%{!?_licensedir:%global license %doc}

%files -n kubelet
%license LICENSE THIRD_PARTY_LICENSES.txt
%{_bindir}/kubelet
%{_sysconfdir}/sysconfig/kubelet
%{_sysconfdir}/systemd/system/kubelet.service
%{_sysconfdir}/systemd/system/kubelet.service.d/10-kubeadm.conf
%{_sysconfdir}/kubernetes/manifests/
/usr/libexec/kubernetes/kubelet-plugins/volume/exec
%doc *.md
%{_mandir}/man1/kubelet*

%files -n kubeadm
%license LICENSE THIRD_PARTY_LICENSES.txt
%doc *.md
%{_mandir}/man1/kubeadm*
%{_bindir}/kubeadm
%{_bindir}/kubeadm-setup.sh
%{_bindir}/kubeadm-registry.sh
/etc/sysctl.d/k8s.conf
/usr/local/share/kubeadm/kubernetes-dashboard-self-certs.yaml
/usr/local/share/kubeadm/flannel-ol.yaml

%files -n kubectl
%license LICENSE THIRD_PARTY_LICENSES.txt
%{_bindir}/kubectl
%{_sysconfdir}/systemd/system/kubectl-proxy.service.d/10-kubectl-proxy.conf
%{_sysconfdir}/systemd/system/kubectl-proxy.service
%doc *.md
%{_mandir}/man1/kubectl*
%{_datadir}/bash-completion/completions/kubectl

%files -n kubeadm-container-images
%license LICENSE THIRD_PARTY_LICENSES.txt
/usr/local/share/kubeadm/kube-apiserver.tar
/usr/local/share/kubeadm/kube-controller-manager.tar
/usr/local/share/kubeadm/kube-scheduler.tar
/usr/local/share/kubeadm/kube-proxy.tar
/usr/local/share/kubeadm/pause.tar

%pre -n kubeadm
# check if this is an upgrade
if [ "$1" == "2" ]; then
    #check if we are upgrading from earlier than 1.11
    # upgrade
    RPM_VERSION=`/bin/rpm -q --queryformat='%{VERSION}' kubeadm 2>&1`
    _major_version=`echo $RPM_VERSION | /bin/awk -F '.' '{print $2}'`
    # any version prior to 1.11 are not compatible
    if [ "${_major_version}" -lt "11" ]; then
        if [ -z "${FORCE_UPDATE_12_FROM_9}" ]; then
            echo 'Can not upgrade kubeadm from version prior to 1.11 due to upgrade compatibility'
            exit 1
        fi
        echo "Upgrading kubeadm forcefully from version earlier that 1.11"
    fi
    rm -rf /usr/local/share/kubeadm/flannel-ol-old.yaml
    cp /usr/local/share/kubeadm/flannel-ol.yaml /usr/local/share/kubeadm/flannel-ol-old.yaml
fi

%post -n kubelet
%systemd_post kubelet
if [ $1 -gt 1 ] ; then
  systemctl daemon-reload >/dev/null 2>&1
  systemctl try-restart kubelet >/dev/null 2>&1
fi

%preun -n kubelet
%systemd_preun kubelet

%postun -n kubelet
%systemd_postun_with_restart kubelet

%changelog
* Wed Jul 15 2020 Thomas Tanaka <thomas.tanaka@orcle.com> - 1.12.10-1.0.13
- CVE-2020-8559: Privilege escalation from compromised node to cluster
- CVE-2020-8557: Node disk DOS by writing to container /etc/hosts

* Wed Jun 03 2020 Thomas Tanaka <thomas.tanaka@oracle.com> - 1.12.10-1.0.12
- CVE-2020-10749: IPv4 only clusters susceptible to MitM attacks via IPv6 rogue router advertisements
- CVE-2020-8555: Half-Blind SSRF in kube-controller-manager

* Tue Apr 07 2020 Thomas Tanaka <thomas.tanaka@oracle.com> - 1.12.10-1.0.11
- [CVE-2019-11254] kube-apiserver Denial of Service vulnerability from malicious YAML payloads 

* Mon Nov 04 2019 Thomas Tanaka <thomas.tanaka@oracle.com> - 1.12.10-1.0.10
- [CVE-2019-16276] Kubernetes Vulnerabilities Allow Authentication Bypass, DoS

* Mon Oct 28 2019 Tom Cocozzello <tom.cocozzello@oracle.com> - 1.12.10-1.0.9
- Define rolling update for flannel

* Mon Oct 21 2019 Tom Cocozzello <tom.cocozzello@oracle.com> - 1.12.10-1.0.8
- Modify flannel/dashboard image tags to use images that have the cve fix

* Wed Oct 16 2019 Tom Cocozzello <tom.cocozzello@oracle.com> - 1.12.10-1.0.7
- [CVE-2019-11253] Kubernetes API Server JSON/YAML parsing vulnerable to resource exhaustion attack

* Tue Oct 15 2019 Tom Cocozzello <tom.cocozzello@oracle.com> - 1.12.10-1.0.6
- [CVE-2019-16276] bump golang to 1.12.10

* Fri Oct 11 2019 Durai G <durai.vattakalvalasu.govindas@oracle.com> - 1.12.10-1.0.5
- added THIRD_PARTY_LICENSES.txt file

* Thu Sep 26 2019 Michael Calunod <michael.calunod@oracle.com> - 1.12.10-1.0.4
- fix for CVE-2019-11251

* Fri Aug 30 2019 Michael Calunod <michael.calunod@oracle.com> - 1.12.10-1.0.3
- replacing references to kubernetes-dashboard-amd64 with kubernetes-dashboard

* Fri Jul 26 2019 Olcne-Builder Jenkins <olcne-builder_us@oracle.com> - 1.12.10-1.0.2
- Added Oracle specific build files for Kubernetes
